"use strict";
var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.stateful = exports.StatefulLifetime = void 0;
var ILifetime_1 = require("./ILifetime");
var StatefulLifetime = /** @class */ (function () {
    function StatefulLifetime(name, factory) {
        Object.defineProperty(this, "name", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: void 0
        });
        Object.defineProperty(this, "factory", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: void 0
        });
        Object.defineProperty(this, "next", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: void 0
        });
        this.name = name;
        this.factory = factory;
        this.next = function (_a) {
            var instances = _a.instances;
            instances[name] = instances[name] || 1;
            return instances[name]++;
        };
    }
    Object.defineProperty(StatefulLifetime.prototype, "provide", {
        enumerable: false,
        configurable: true,
        writable: true,
        value: function (provider, context) {
            var _this = this;
            var _a;
            var parentContext = context.parent;
            var isSingleton = ((_a = context.lastSingleton) !== null && _a !== void 0 ? _a : {}).isSingleton;
            var name = this.name;
            var next = this.next;
            function createContext() {
                if (!parentContext.depth || !parentContext.isEscaped)
                    return provider.enterLifetime(name, parentContext, ILifetime_1.Lifetime.dummy("".concat(name, "@instance"), isSingleton));
                var id = next(parentContext);
                return provider.enterLifetime(name, undefined, ILifetime_1.Lifetime.dummy("".concat(name, "@instance#").concat(id), isSingleton));
            }
            return {
                create: function (props) {
                    var usedContext = createContext();
                    var result = _this.factory(provider.createProxy(name, usedContext), props, provider, usedContext);
                    usedContext.isEscaped = true;
                    return result;
                },
            };
        }
    });
    Object.defineProperty(StatefulLifetime.prototype, "clone", {
        enumerable: false,
        configurable: true,
        writable: true,
        value: function () {
            return new StatefulLifetime(this.name, this.factory);
        }
    });
    return StatefulLifetime;
}());
exports.StatefulLifetime = StatefulLifetime;
function stateful(dep) {
    return (typeof dep === 'function'
        ? { lifetime: StatefulLifetime, constructor: dep }
        : __assign({ lifetime: StatefulLifetime }, dep));
}
exports.stateful = stateful;
